#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#ifdef __APPLE__
#include <GLUT/glut.h>
#elif defined(WIN32)
#include "glut.h"
#else
#include <GL/glut.h>
#endif

#include "image_rgba.h"

void disp(void);
void idle(void);
void draw_chess(int xdiv, int ydiv);
void reshape(int x, int y);
void keyb(unsigned char key, int x, int y);
unsigned int load_texture(const char *fname);

unsigned int tex;

int main(int argc, char **argv)
{
	glutInit(&argc, argv);
	glutInitWindowSize(800, 600);
	glutInitDisplayMode(GLUT_RGB | GLUT_DEPTH | GLUT_DOUBLE);
	glutCreateWindow("TGA loading test");

	glutDisplayFunc(disp);
	glutIdleFunc(idle);
	glutReshapeFunc(reshape);
	glutKeyboardFunc(keyb);

	if(!(tex = load_texture(argv[1] ? argv[1] : "test.tga"))) {
		return 1;
	}

	glutMainLoop();
	return 0;
}

void disp(void)
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	draw_chess(32, 32);

	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, tex);

	glBegin(GL_QUADS);
	glColor4f(1, 1, 1, 1);
	glTexCoord2f(0, 1);
	glVertex2f(-1, -1);
	glTexCoord2f(1, 1);
	glVertex2f(1, -1);
	glTexCoord2f(1, 0);
	glVertex2f(1, 1);
	glTexCoord2f(0, 0);
	glVertex2f(-1, 1);
	glEnd();

	glDisable(GL_TEXTURE_2D);

	glDisable(GL_BLEND);

	glutSwapBuffers();
}

void idle(void)
{
	glutPostRedisplay();
}

/* just a backdrop to show through transparent images */
void draw_chess(int xdiv, int ydiv)
{
	static const float colors[][4] = {
		{0.6, 0.5, 0.5, 1.0},
		{0.7, 0.7, 0.8, 1.0}
	};
	float dx = 2.0 / xdiv;
	float dy = 2.0 / ydiv;
	float x, y, aspect;
	int i, j, vp[4];
	float t = glutGet(GLUT_ELAPSED_TIME) / 1000.0;

	glGetIntegerv(GL_VIEWPORT, vp);
	aspect = (float)(vp[2] - vp[0]) / (float)(vp[3] - vp[1]);

	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
	glOrtho(-aspect, aspect, -1, 1, -1, 1);

	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity();
	glTranslatef(cos(t / 2.0) * 0.5, sin(t) * 0.35, 0);
	glScalef(2, 2, 2);

	glBegin(GL_QUADS);

	x = -1.0f;
	for(i=0; i<xdiv; i++) {
		y = -1.0f;
		for(j=0; j<ydiv; j++) {
			glColor3fv((i & 1) == (j & 1) ? colors[0] : colors[1]);

			glVertex2f(x, y);
			glVertex2f(x + dx, y);
			glVertex2f(x + dx, y + dy);
			glVertex2f(x, y + dy);

			y += dy;
		}
		x += dx;
	}

	glEnd();

	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();
}

void reshape(int x, int y)
{
	glViewport(0, 0, x, y);
}

void keyb(unsigned char key, int x, int y)
{
	if(key == 27) {
		exit(0);
	}
}

/* uses the TGA version of load image to load a TARGA image file
 * and create an OpenGL texture out of it
 */
unsigned int load_texture(const char *fname)
{
	unsigned int tex;
	struct image_rgba *img;

	if(!(img = load_image_rgba(fname))) {
		return 0;
	}

	glGenTextures(1, &tex);
	glBindTexture(GL_TEXTURE_2D, tex);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);

	/* notice the use of GL_RGBA instead of GL_RGB, since we have alpha values in the image */
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, img->width, img->height, 0, GL_RGBA, GL_UNSIGNED_BYTE, img->pixels);

	free_image_rgba(img);
	return tex;
}
